<?php
namespace Teknisa\PLA\Modules\Prato\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet as DataSet_Source;

class RevisarReceita extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $pratoService;
    protected $prato;
    protected $environment;

    public function __construct(
        $crudService, $parameterBag,
        \Teknisa\PLA\Modules\Prato\Services\PratoService $pratoService,
        \Teknisa\PLA\Modules\Prato\Controllers\Prato $prato,
        \Teknisa\GER\Utilities\Environment $environment
    ) {
        parent::__construct($crudService, $parameterBag);
        $this->pratoService = $pratoService;
        $this->prato = $prato;
        $this->environment = $environment;
    }


    public function revisarReceita(
        \Zeedhi\Framework\DTO\Request\Row $request,
        \Zeedhi\Framework\DTO\Response $response
    ){
        try {
            $this->pratoService->iniciaTransacao();

            $row = $request->getRow();
            $tpImportacao = $row['TIPO_IMPORT'];
            $idProporcao = $row['ID_PROPORCAO'];
            $recOrigem = $row['RECORIGEM'];
            if ($recOrigem == "L") {
                $filialO = $row['CDFILIAL_O'];
                $servicoO = 'X';
            } else if ($recOrigem == "S") {
                $filialO = $row['CDFILIAL_O'];
                $servicoO = $row['CDSERVFILI_O'];
            } else {
                $filialO = 'X';
                $servicoO = 'X';
            }

            $recDestino = $row['RECDESTINO'];
            if ($recDestino == "L") {
                $filialD = $row['CDFILIAL_D'];
                $servicoD = 'X';
            } else if ($recDestino == "S") {
                $filialD = $row['CDFILIAL_D'];
                $servicoD = $row['CDSERVFILI_D'];
            } else {
                $filialD = 'X';
                $servicoD = 'X';
            }

            $nrorg = $this->environment->getNrOrgTrab();
            $operador = $this->environment->getCdOperador();

            foreach ($row['PRATOS'] as $pratos) {
                // Exclui Receita se for sobrepor
                if ($tpImportacao === 'S') {
                    if ($recDestino === 'P') {
                        $receitaD = $this->pratoService->retornaReceitaPadrao(
                            $pratos['CDPRATO'], $pratos['CDFILAUXPRAT']
                        );
                    } else if ($recDestino === 'L') {
                        $receitaD = $this->pratoService->retornaReceitaLocal(
                            $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'], $filialD
                        );
                    } else if ($recDestino === 'S') {
                        $receitaD = $this->pratoService->retornaReceitaServico(
                            $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                            $filialD, $servicoD
                        );
                    }

                    if ($receitaD) {
                        foreach ($receitaD as $produtos) {
                            $this->pratoService->excluiReceita($produtos);
                        }
                    }
                }

                //Faz a alteração porporcional se identificador marcado
                if ($idProporcao === 'S'){
                    $proporcao = $pratos['PROPORCAO'];
                } else {
                    $proporcao = 1;
                }

                // Busca dados da receita de origem para atualizar os campos de
                // cozinha central
                $idFinal   = $qtLim    = $idCook   = $idTpCook   = $prodProd = null;
                $corPri    = $corSeq   = $corAux   = $quantidade = $prodProd = null;
                $container = $hora     = $nomeFant = $qtCarb     = null;
                $qtLip     = $qtProt   = $qtCal    = $qtAlc      = $qtMin    = null;
                $qtAmost   = $idCook   = $idFinal  = $qtLim      = $idTpCook = null;

                if ($recOrigem === 'P') {
                    $dadosPratO = $this->pratoService->retornaPratPadr(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT']
                    );

                    if ($dadosPratO instanceof \Teknisa\Model\Planejamento\Entities\Pratpadr) {
                        $idFinal = $dadosPratO->getIdfinalpadr();
                        $qtLim = $dadosPratO->getQtlimsegpadr();
                        $idCook = $dadosPratO->getIdprodchill();
                        $idTpCook = $dadosPratO->getIdtpprodchill();
                        $prodProd = $dadosPratO->getCdprodproducao();

                        $corPri = $dadosPratO->getNrcorpriprat();
                        $corSeq = $dadosPratO->getNrcorseqprat();
                        $corAux = $dadosPratO->getNrcorauxprat();
                        $quantidade = $dadosPratO->getQtpadraoprat();
                        $container = $dadosPratO->getQtcontaiprat();
                        $hora = $dadosPratO->getHrvalconprat();
                        $nomeFant = $dadosPratO->getNmfantasiapr();
                        $qtCarb = $dadosPratO->getQtcarboprpad();
                        $qtLip = $dadosPratO->getQtlipidprpad();
                        $qtProt = $dadosPratO->getQtproteprpad();
                        $qtCal = $dadosPratO->getQtcalorprpad();
                        $qtAlc = $dadosPratO->getQtalcooprpad();
                        $qtMin = $dadosPratO->getVrpeminpadr();
                        $qtAmost = $dadosPratO->getQtamostrpadr();
                    }

                } else if ($recOrigem === 'L') {
                    
                    $dadosPratO = $this->pratoService->retornaPratFili(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'], $filialO
                    );
                      
                    if ($dadosPratO instanceof \Teknisa\Model\Planejamento\Entities\Pratfili) {

                        $idFinal = $dadosPratO->getIdfinalfili();
                        $qtLim = $dadosPratO->getQtlimsegfili();
                        $idCook = $dadosPratO->getIdprodchill();
                        $idTpCook = $dadosPratO->getIdtpprodchill();
                        $prodProd = $dadosPratO->getCdprodproducao();

                        $corPri = $dadosPratO->getNrcorprifili();
                        $corSeq = $dadosPratO->getNrcorseqfili();
                        $corAux = $dadosPratO->getNrcorauxfili();
                        $quantidade = $dadosPratO->getQtpadraofili();
                        $container = $dadosPratO->getQtcontaifili();
                        $hora = $dadosPratO->getHrvalconfili();
                        $nomeFant = $dadosPratO->getNmfantasiapf();
                        $qtCarb = $dadosPratO->getQtcarboprfil();
                        $qtLip = $dadosPratO->getQtlipidprfil();
                        $qtProt = $dadosPratO->getQtproteprfil();
                        $qtCal = $dadosPratO->getQtcalorprfil();
                        $qtAlc = $dadosPratO->getQtalcooprfil();
                        $qtMin = $dadosPratO->getVrpeminfili();
                        $qtAmost = $dadosPratO->getQtamostrfili();
                    }

                } else {
                      
                    $dadosPratO = $this->pratoService->retornaPratServ(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                        $filialO, $servicoO
                    );

                    if ($dadosPratO instanceof \Teknisa\Model\Planejamento\Entities\Pratserv) {
                        $idFinal = $dadosPratO->getIdfinalserv();
                        $qtLim = $dadosPratO->getQtlimsegserv();
                        $idCook = $dadosPratO->getIdprodchill();
                        $idTpCook = $dadosPratO->getIdtpprodchill();
                        $prodProd = $dadosPratO->getCdprodproducao();

                        $corPri = $dadosPratO->getNrcorpriserv();
                        $corSeq = $dadosPratO->getNrcorseqserv();
                        $corAux = $dadosPratO->getNrcorauxserv();
                        $quantidade = $dadosPratO->getQtpadraoserv();
                        $container = $dadosPratO->getQtcontaiserv();
                        $hora = $dadosPratO->getHrvalconserv();
                        $nomeFant = $dadosPratO->getNmfantasiase();
                        $qtCarb = $dadosPratO->getQtcarboprser();
                        $qtLip = $dadosPratO->getQtlipidprser();
                        $qtProt = $dadosPratO->getQtproteprser();
                        $qtCal = $dadosPratO->getQtcalorprser();
                        $qtAlc = $dadosPratO->getQtalcooprser();
                        $qtMin = $dadosPratO->getVrpeminserv();
                        $qtAmost = $dadosPratO->getQtamostrserv();
                    }
                }

                if ($tpImportacao === 'N') {
                    $idCook = null;
                    $idFinal = 'M';
                    $qtLim = null;
                    $idTpCook = null;
                    $prodProd = null;
                }
                $idFinal = $idFinal ? $idFinal : 'M';
                if ($pratos['QUANTIDADE'] <> $pratos['QTDANTERIOR']){
                    $quantidade = str_replace(',', '.', $pratos['QUANTIDADE']);
                }

                // Adiciona os produtos da receita espelho e
                // exclui dados do prato de destino e já insere com os dados
                // de origem
                if ($recDestino === 'P') {
                    $this->pratoService->revisaReceitaPadrao(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                        $recOrigem, $filialO, $servicoO, $proporcao,
                        $nrorg, $operador
                    );

                    $pratpadrD = $this->pratoService->retornaPratPadr(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT']
                    );
                    $this->pratoService->excluiDadosReceita($pratpadrD);
                    
                          //caso esteja acresentando, o produto de produção deve se nulo
                    $tpImportacao === 'A' ? $prodProd=null: 0;
                    $this->prato->insereDadosPrato(
                        $pratos['CDFILAUXPRAT'], $pratos['CDPRATO'], null, null,
                        $corPri, $corSeq, $corAux, $quantidade, $container,
                        $hora, $nomeFant, $qtCarb, $qtLip, $qtProt, $qtCal,
                        $qtAlc, $qtMin, $qtAmost, $prodProd, $qtLim, $idCook,
                        $idTpCook, $idFinal, $nrorg
                    );
                } else if ($recDestino === 'L') {
                    $this->pratoService->revisaReceitaLocal(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                        $recOrigem, $filialO, $servicoO, $filialD, $proporcao,
                        $nrorg, $operador
                    );
                    
                    $pratLocalD = $this->pratoService->retornaPratFili(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'], $filialD
                    );
                    
                          //caso esteja acresentando, o produto de produção deve se nulo
                    $tpImportacao === 'A' ? $prodProd=null: 0;
                    
                    
                    $this->pratoService->excluiDadosReceita($pratLocalD);
                
                    
                    $this->prato->insereDadosPrato(
                        $pratos['CDFILAUXPRAT'], $pratos['CDPRATO'], $filialD, null,
                        $corPri, $corSeq, $corAux, $quantidade, $container,
                        $hora, $nomeFant, $qtCarb, $qtLip, $qtProt, $qtCal,
                        $qtAlc, $qtMin, $qtAmost, $prodProd, $qtLim, $idCook,
                        $idTpCook, $idFinal, $nrorg
                    );
                    
                } else if ($recDestino === 'S') {
                    $this->pratoService->revisaReceitaServico(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                        $recOrigem, $filialO, $servicoO, $proporcao,
                        $filialD, $servicoD, $nrorg, $operador
                    );

                    $pratServicoD = $this->pratoService->retornaPratServ(
                        $pratos['CDPRATO'], $pratos['CDFILAUXPRAT'],
                        $filialD, $servicoD
                    );
                       //caso esteja acresentando, o produto de produção deve se nulo
                    $tpImportacao === 'A' ? $prodProd=null: 0;
                    
                    
                    $this->pratoService->excluiDadosReceita($pratServicoD);

                    $this->prato->insereDadosPrato(
                        $pratos['CDFILAUXPRAT'], $pratos['CDPRATO'], $filialD, $servicoD,
                        $corPri, $corSeq, $corAux, $quantidade, $container,
                        $hora, $nomeFant, $qtCarb, $qtLip, $qtProt, $qtCal,
                        $qtAlc, $qtMin, $qtAmost, $prodProd, $qtLim, $idCook,
                        $idTpCook, $idFinal, $nrorg
                    );
                }
            }
            $this->pratoService->commit();
            $response->addDataset(new DataSet_Source('REVISAO', array(true)));
        } catch (\Exception $e) {
            $this->pratoService->rollback();
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), 1));
        }
    }
    

}